#include "mateconf-keys.h"


namespace procman
{
  namespace mateconf
  {
    const std::string root("/apps/procman");
    const std::string solaris_mode(root + "/solaris_mode");
    const std::string open_files_tree_prefix(root + "/openfilestree");
    const std::string network_in_bits(root + "/network_in_bits");
  }
}

