#ifndef H_PROCMAN_MATECONF_KEYS_1177430397
#define H_PROCMAN_MATECONF_KEYS_1177430397

#include <string>


namespace procman
{
  namespace mateconf
  {
    extern const std::string root;
    extern const std::string solaris_mode;
    extern const std::string open_files_tree_prefix;
    extern const std::string network_in_bits;
  }
}


#endif //  H_PROCMAN_MATECONF_KEYS_1177430397
